/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "valueindicator.h"

#include <limits>
#include <cmath>

#include <QDebug>

ValueIndicator::ValueIndicator(ValueIndicatorDense *dense) : QObject()
  , m_value(0)
  , m_dense(dense)
{

}

qreal ValueIndicator::value() const
{
    return (m_value - m_dense->min())/(m_dense->max() - m_dense->min());
}


ValueIndicatorDense::ValueIndicatorDense(int rows, int columns, const QList<ValueIndicator*>& data) : AbstractDense(rows, columns, data)
  , m_max(std::numeric_limits<double>::min())
  , m_min(std::numeric_limits<double>::max())
{

}

void ValueIndicatorDense::updateValues(const Dense& dense)
{
    m_max = std::numeric_limits<double>::min();
    m_min = std::numeric_limits<double>::max();

    int i = 0;
    for (auto value : m_data) {
        double val = dense.rawValue(i);
        if (val > m_max) {
            m_max = val;
        }

        if (val < m_min) {
            m_min = val;
        }
        value->setValue(val);
        i++;
    }
}

ValueIndicatorDense::~ValueIndicatorDense()
{
    qDeleteAll(m_data);
}
